-- myDebug.e
-- v1.0
-- 2010-11-13
-- debug controls

include file.e
include graphics.e
include image.e
include msgbox.e
include get.e
include myTypes.e

global integer
  f_debug    -- debug file

------------------------------------------------------------------------------

global function dateStamp(sequence msg)
-- prefixes the message by date and time for logging
  sequence cur_date

  cur_date = date()
  return sprintf("%d-%02d-%02d %02d:%02d:%02d -> %s\n", {
                             (cur_date[1] + 1900), cur_date[2], cur_date[3],
                              cur_date[4], cur_date[5], cur_date[6], msg
                            })
end function

------------------------------------------------------------------------------

global procedure logMsg(sequence msg)
-- records logs
  puts(f_debug, dateStamp(msg))
  flush(f_debug)
end procedure

------------------------------------------------------------------------------

function splitMessage(sequence msg, atom sep)
-- splits str string in tokens according to separator sep
  integer slen
  sequence result, s

  result = {}
  slen = length(msg)
  s = {}
  for i = 1 to slen do
    if msg[i] = sep then
      result = append(result, s)
      s = {}
 	else
	  s = s & msg[i]
	end if
  end for
  if length(s) > 0 then result = append(result, s) end if
  return result
end function

-------------------------------------------------------------------------------

procedure displayError(sequence msg)
-- prints a message in a frame at position pos
-- if pos is an atom (mostly 0), then center the frame in the screen
  sequence saved, vc, s
  integer h, w, x1, y1, x2, y2
  atom c

  text_color(BRIGHT_WHITE)
  bk_color(RED)
  vc = video_config()
  s = splitMessage(msg, '\n')
  h = length(s)
  w = 0
  for i = 1 to h do
    if length(s[i]) > w then w = length(s[i]) end if
  end for
  x1 = floor((vc[VC_COLUMNS]-w)/2)
  y1 = floor((vc[VC_LINES]-h)/2)
  x2 = x1+w+1
  y2 = y1+h
  saved = save_text_image({y1,x1}, {y2,x2})
  position(y1,x1)     puts(1, repeat(' ', w+2))
  position(y1+1,x1) puts(1, " "&msg&" ")
  position(y2,x1)     puts(1, repeat(' ', w+2))
  c = wait_key()
  display_text_image({y1,x1}, saved)
end procedure

------------------------------------------------------------------------------

global procedure warnError(sequence msg, atom quit)
-- logs an error and displays a warning
-- if quit is set then abort
  logMsg(msg)
  if platform() = WIN32 then
    void = message_box(msg, "Erreur", MB_OK+MB_ICONERROR)
  else
    displayError(msg)
  end if
  if quit then
    abort(1)
  end if
end procedure

------------------------------------------------------------------------------

global procedure analyzeSequence(object s, sequence variable, integer output)
-- prints a sequence structure in a human readable way
  if integer(s) then
    printf(output, "%s = %d\n", {variable, s})
  elsif atom(s) then
    if (s >= 0) and (s = floor(s)) then
      printf(output, "%s = %.0f\n", {variable, s})
    elsif (s < 0) and (s = floor(s+1)) then
      printf(output, "%s = %.0f\n", {variable, s})
    else
      printf(output, "%s = %f\n", {variable, s})
    end if
  elsif string(s) then
    printf(output, "%s = '%s'\n", {variable, s})
  else
    for i=1 to length(s) do
      analyzeSequence(s[i], variable&sprintf("[%d]", i), output)
    end for
  end if
  if output=f_debug then flush(f_debug) end if
end procedure


